/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import java.util.List;

/**
 * An Event belongs to a ReactiveCommunity. An Event occurs or doesn't occur
 * during a particular instant of its ReactiveCommunity.
 * A base Event (not a message Event) is atomic and instantaneous:
 * it is obtained at the same instant of its generation.
 * A MessageEvent is an Event that can have attached Messages. It is
 * broadcasted at the beginning of the instant that just follows the instant of
 * its generation.
 * An Event may have an unique destination. It is said to be exclusive.
 */
public interface Event {

	/**
	 * @return the ReactiveCommunity to which this Event belongs to,
	 */
	public ReactiveCommunity getCommunity();

	/**
	 * @return true if this Event has an unique destination, false elsewhere.
	 */
	public boolean isExclusive();
	
	/**
	 * @return the id of the unique destination entity if any, null elsewhere.
	 */
	public String getUniqueDestinationEntityId();
	
	/**
	 * @return the list of messages that can be read at this instant if this
	 *         Event is a message Event, null elsewhere.
	 */
	public List<Message> getMessageList();
}
