/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import java.util.List;

/**
 * A ReactiveAgent reacts to events and messages.
 */
public interface ReactiveAgent {

	/**
	 * Reacts to an elementary global event.
	 * 
	 * @param event
	 *            an elementary Event.
	 * @return false if this ReactiveAgent must terminate.
	 */
	public boolean reactToElementaryEvent(Event event);

	/**
	 * Reacts to a not exclusive message Event.
	 * 
	 * @param messageEvent
	 *            a not exclusive message Event.
	 * @param messageList
	 *            a list of Message.
	 * @return false if this ReactiveAgent must terminate.
	 */
	public boolean reactToGlobalMessageEvent(Event messageEvent,
			List<Message> messageList);

	/**
	 * Reacts to an exclusive Message that is not destined to a particular Cell
	 * (if any).
	 * 
	 * @param message
	 *            an exclusive Message that is not destined to a particular Cell
	 *            (if any).
	 * @return false if this ReactiveAgent must terminate.
	 */
	public boolean reactToExclusiveMessage(Message message);

	/**
	 * Reacts to a Message that is destined to a particular Cell (if any).
	 * 
	 * @param message
	 *            a Message that is destined to a particular Cell (if any).
	 */
	public void reactToCellMessage(Message message);

	/**
	 * A new instant occurs.
	 * 
	 * @return false if this ReactiveAgent must terminate.
	 */
	public boolean newInstant();
}
