/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import org.xmloperator.reactive.exception.AgentAlreadyLaunchedException;
import org.xmloperator.reactive.exception.DoneMachineException;
import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * The task of a ReactiveMachine consists in launching agents.
 * Methods of this Object must accept to be called from multiple Threads. 
 */
public interface ReactiveMachine {

	/**
	 * Attach an ActivityObserver to this ReactiveMachine or detach it.
	 * @param activityObserver an ActivityObserver or null.
	 */
	public void setActivityObserver(ActivityObserver activityObserver);

	/**
	 * Creates and returns a new ReactiveCommunity.
	 * It will be associated to this ReactiveMachine.
	 * @return a new ReactiveCommunity.
	 */
	public ReactiveCommunity newCommunity();
	
	/**
	 * Launches an Agent into a community.
	 * 
	 * @param agent
	 *            an Agent
	 * @param reactiveCommunity
	 *            a destination ReactiveCommunity.
	 * @return the entityId of the Agent.
	 * @exception AgentAlreadyLaunchedException
	 *                if the given Agent is already running.
	 * @exception DoneMachineException
	 *                thrown if this ReactiveMachine is done.
	 */
	public String launchAgent(Agent agent, ReactiveCommunity reactiveCommunity)
			throws AgentAlreadyLaunchedException, DoneMachineException;

	/**
	 * Waits for all agents terminate, then terminates this ReactiveMachine.
	 * 
	 * @param machineDoneKey
	 *            a key that restricts the usage of this method.
	 * @exception IncorrectKeyException
	 *                if the machineDoneKey value is not the good one.
	 */
	public void done(long machineDoneKey) throws IncorrectKeyException;
}
