/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.xmloperator.reactive.ActivityObserver;
import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.AgentActivity;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveEntity;

/**
 * ActivityObserver implementation.
 */
public class ActivityObserverImpl extends Thread implements ActivityObserver {
	
	private static PrintStream trace = null;

	public static void setTrace(PrintStream trace) {
		ActivityObserverImpl.trace = trace;
	}
		
	protected final int period_ms = 7;
	protected final List<AgentActivity> agentActivityList = new ArrayList<AgentActivity>();
	protected final Map<Agent, AgentActivity> fromAgentToAgentActivity = new HashMap<Agent, AgentActivity>();
	protected final Set<AgentActivity> participatingAgentActivities = new HashSet<AgentActivity>();
	protected final List<AgentActivity> tempList = new ArrayList<AgentActivity>();
	protected final Object synchroTerminate = new Object();
	protected boolean isRunning = true;
	protected int stepCount = 0;

	public ActivityObserverImpl() {
		super("Observer");
		this.setPriority(Thread.NORM_PRIORITY + 2);
	}
	
	public void terminate() {
		this.isRunning = false;
		try {
			synchronized (this.synchroTerminate) {
				this.synchroTerminate.wait();
			}
		}
		catch (InterruptedException exception) {}
		if (trace != null) {
			trace.println("--- Agent activities:");
			for (AgentActivity agentActivity : this.agentActivities()) {
				trace.println(agentActivity.toString());
			}
		}
	}
	
	// ////////////////////////////////
	// ActivityObserver implementation
	// ////////////////////////////////

	public void notifyAgentStarting(Agent agent) {
		AgentActivity agentActivity = new AgentActivityImpl(agent);
		synchronized (this.participatingAgentActivities) {
			this.agentActivityList.add(agentActivity);
			this.fromAgentToAgentActivity.put(agent, agentActivity);
			this.participatingAgentActivities.add(agentActivity);
		}
	}

	public void notifyAgentTerminating(Agent agent) {
		AgentActivity agentActivity = (AgentActivity) this.fromAgentToAgentActivity
				.get(agent);
		synchronized (this.participatingAgentActivities) {
			this.participatingAgentActivities.remove(agentActivity);
		}
	}

	public void clearAgentActivities() {
		this.stepCount = 0;
		for (AgentActivity agentActivity : this.agentActivityList) {
			((AgentActivityImpl) agentActivity).activity = AgentActivityImpl.RESIDUAL_ACTIVITY;
		}
	}
	
	public AgentActivity[] agentActivities() {
		synchronized (this.participatingAgentActivities) {
			int agentActivityCount = this.agentActivityList.size();
			AgentActivity[] activities = new AgentActivity[agentActivityCount];
			for (int k = 0; k < agentActivityCount; k++) {
				AgentActivityImpl agentActivity = (AgentActivityImpl)this.agentActivityList.get(k);
				AgentActivityImpl clonedAgentActivity = new AgentActivityImpl(agentActivity);
				if (this.stepCount != 0)
					clonedAgentActivity.activity /= this.stepCount;
				activities[k] = clonedAgentActivity;
			}
			return activities;
		}
	}

	// //////////////////////
	// Thread implementation
	// //////////////////////

	public void run() {
		while (this.isRunning) {
			synchronized (this.participatingAgentActivities) {
				for (AgentActivity agentActivity : this.participatingAgentActivities) {
					AgentImpl agent = (AgentImpl) agentActivity.getAgent();
					if (agent.getStatus() == LaunchingContext.STATUS_PARTICIPATING) {
						ReactiveEntity entity = agent.getReactiveEntity();
						if (entity.getStatus() == ReactiveEntity.STATUS_ACTIVE)
							this.tempList.add(agentActivity);
					}
				}
			}
			this.stepCount++;
			if (!this.tempList.isEmpty()) {
				float activity = ((float) 1.0) / this.tempList.size();
				for (AgentActivity agentActivity : this.tempList)
					((AgentActivityImpl) agentActivity).activity += activity;
				this.tempList.clear();
			}
			try {
				Thread.sleep(this.period_ms);
			} catch (InterruptedException exception) {}
		}
		synchronized (this.synchroTerminate) {
			this.synchroTerminate.notifyAll();
		}
	}
}
