/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.CrashingReport;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveEntity;
import org.xmloperator.reactive.StartingAndTerminationNotifier;

final class AgentThread extends Thread {
	
	private static PrintStream trace = null;
	private static boolean isFullTrace = false;
	
	public static void setTrace(PrintStream trace, boolean isFullTrace) {
		AgentThread.trace = trace;
		AgentThread.isFullTrace = isFullTrace;
	}

	private final StartingAndTerminationNotifier<Agent> agentNotifier;
	private final Agent agent;
	private final LaunchingContextImpl launchingContext;
	private final CrashingReport report;
	
	public AgentThread(
			StartingAndTerminationNotifier<Agent> agentNotifier,
			Agent agent,
			LaunchingContextImpl launchingContext,
			CrashingReport report) {
		super("agent");
		if (agentNotifier == null || agent == null || launchingContext == null)
			throw new IllegalArgumentException();
		this.setPriority(Thread.NORM_PRIORITY);
		this.agentNotifier = agentNotifier;
		this.agent = agent;
		this.launchingContext = launchingContext;
		this.report = report;
	}
	
	private final void crash(String entityId, String circumstance,
			Exception exception) {
		if (trace != null)
			trace.println(entityId + " crashes when " + circumstance);
		if (this.report != null)
			this.report.notifyCrash(entityId, exception);
		else
			exception.printStackTrace();
	}

	////////////////////////
	// Thread implementation
	////////////////////////
	
	public void run() {
		ReactiveEntity reactiveEntity = this.launchingContext.getReactiveEntity();
		reactiveEntity.setThread(this);
		if (trace != null)
			trace.println(reactiveEntity + " is " + this.agent);
		String entityId = reactiveEntity.getEntityId();
		this.setName(entityId);
		this.agentNotifier.notifyStarting(this.agent);
		try {
			if (trace != null && isFullTrace)
				trace.println(entityId + " init");
			this.launchingContext.setAgentStatus(LaunchingContext.STATUS_INITIALIZING);
			this.agent.init(this.launchingContext);
			this.launchingContext.setAgentStatus(LaunchingContext.STATUS_PARTICIPATING);
			try {
				this.agent.participate();
			} catch (Exception exception) {
				this.crash(entityId, "participating", exception);
			}
			this.launchingContext.setAgentStatus(LaunchingContext.STATUS_TERMINATING);
			this.agent.terminate();
			if (trace != null && isFullTrace)
				trace.println(entityId + " terminate");
		} catch (Exception exception) {
			this.crash(entityId, "initializing or terminating", exception);
		}
		this.agentNotifier.notifyTermination(this.agent);
	}
}
