/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import java.util.List;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveAgent;
import org.xmloperator.reactive.exception.ReactiveException;

public abstract class ReactiveAgentImpl extends AgentImpl implements ReactiveAgent {

	public ReactiveAgentImpl(boolean createExclusiveMessageEventOnConnect) {
		super(createExclusiveMessageEventOnConnect);
	}

	///////////////////////////////
	// ReactiveAgent implementation
	///////////////////////////////
	
	public boolean reactToElementaryEvent(Event event) {
		return true;
	}

	public boolean reactToGlobalMessageEvent(Event messageEvent,
			List<Message> messageList) {
		return true;
	}

	public boolean reactToExclusiveMessage(Message message) {
		return true;
	}

	public void reactToCellMessage(Message message) {
		throw new ReactiveException("Unknown cell");
	}

	public boolean newInstant() {
		return true;
	}

	///////////////////////
	// Agent implementation
	///////////////////////
	
	public final void participate() {
		while (true) {
			List<Event> eventList = this.entity.suspend();
			if (eventList.isEmpty()) {
				boolean isOk = this.newInstant();
				if (!isOk)
					return;
				continue;
			}
			for (Event event : eventList) {
				List<Message> messageList = event.getMessageList();
				if (messageList == null) {
					boolean isOk = this.reactToElementaryEvent(event);
					if (!isOk)
						return;
				} else if (event.isExclusive()) {
					for (Message message : messageList) {
						if (message.getDestinationCellNumber() < 0) {
							boolean isOk = this.reactToExclusiveMessage(message);
							if (!isOk)
								return;
						} else
							this.reactToCellMessage(message);
					}
				} else {
					boolean isOk = this.reactToGlobalMessageEvent(event,
							messageList);
					if (!isOk)
						return;
				}
			}
		}
	}
}
