/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.event;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.agent.AgentImpl;

/**
 * The purpose of this Agent consists in listening the first cycle of generated
 * events.
 */
public final class EventListenerAgent extends AgentImpl {
	private final int eventGeneratorCount;
	private final int instantCount;
	
	private Event exitEvent = null;
	private Map<Event, Integer> fromEventToAgentIndex = new Hashtable<Event, Integer>();
	private Map<String, Count> fromOrderToCount = new Hashtable<String, Count>();
	private List<String> orderList = new ArrayList<String>();

	public EventListenerAgent(int eventGeneratorCount, int instantCount) {
		super(false);
		if (eventGeneratorCount < 0 || instantCount < 0)
			throw new IllegalArgumentException();
		this.eventGeneratorCount = eventGeneratorCount;
		this.instantCount = instantCount;
	}

	// /////////////////////
	// Agent implementation
	// /////////////////////

	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		ReactiveCommunity community = launchingContext.getReactiveCommunity();
		this.exitEvent = community.createEvent();
		for (int k = 0; k < this.eventGeneratorCount; k++) {
			Event event = community.createEvent();
			this.fromEventToAgentIndex.put(event, new Integer(k));
			Agent eventGenerator = new EventGeneratorAgent(event,
					this.exitEvent);
			community.launchAgent(eventGenerator);
		}
		this.entity.stop();
	}

	public void participate() {
		for (int k = 0; k < this.instantCount; k++) {
			List<Event> eventList = this.entity.suspend();
			if (eventList.isEmpty())
				throw new IllegalStateException();
			StringBuffer sb = new StringBuffer();
			for (Event e : eventList) {
				int agentIndex = this.fromEventToAgentIndex.get(e).intValue();
				sb.append((char)('a' + agentIndex));
			}
			String order = sb.toString();
			Count count = this.fromOrderToCount.get(order);
			if (count == null) {
				count = new Count();
				this.fromOrderToCount.put(order, count);
				this.orderList.add(order);
			}
			count.count++;
			this.entity.stop();
		}
	}

	public void terminate() {
		this.entity.generateEvent(this.exitEvent);
		super.terminate();
		if (this.trace != null)
			for (int k = 0; k < this.orderList.size(); k++) {
				String order = this.orderList.get(k);
				int count = this.fromOrderToCount.get(order).count;
				StringBuffer sb = new StringBuffer();
				sb.append(order);
				sb.append(' ');
				for (int n = 0; n < count; n++)
					sb.append('*');
				this.trace.println(sb.toString());
			}
	}
}
