/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land;

/**
 * An AgentLocation maintains the location of an Agent in a graph of stations.
 */
public interface AgentLocation {

	/**
	 * @return the AgentBody of the Agent.
	 */
	public AgentBody getAgentBody();

	/**
	 * @return the current Station of the Agent or null if it is not located.
	 */
	public Station getCurrentStation();

	/**
	 * Returns the future Station of the Agent, as determined by a
	 * NavigationStrategy.
	 * 
	 * @param navigationStrategy
	 *            a NavigationStrategy
	 * @return the future Station of the Agent, as determined by the given
	 *         NavigationStrategy.
	 */
	public Station getFutureStation(NavigationStrategy navigationStrategy);
	
	/**
	 * Moves the Agent to a Station, which must be connected to the current one,
	 * if any.
	 * 
	 * @param destination
	 *            a destination Station.
	 * @return the Move if any, null elsewhere.
	 */
	public Move move(Station destination);

	/**
	 * Quits any location. This is irreversible.
	 * 
	 * @return the Move if any, null elsewhere.
	 */
	public Move quit();
	
	/**
	 * @return the visit memory of the Agent.
	 */
	public VisitMemory getVisitMemory();
}
