/*
 * Copyright (c) 2008, 2011 The_xmloperator_project. All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land;

import org.xmloperator.reactive.Board;
import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * Board of a land. The land is a Graph of stations occupied by mobile agents.
 */
public interface LandBoard extends Board {

	/**
	 * An Agent appears.
	 * 
	 * @param boardPermissionKey
	 *            the board permission key.
	 * @param agentBody
	 *            the AgentBody of the Agent.
	 * @throws IncorrectKeyException
	 *             if the given permission key is not the good one.
	 */
	public void agentAppear(long boardPermissionKey, AgentBody agentBody)
			throws IncorrectKeyException;

	/**
	 * An Agent terminates.
	 * 
	 * @param boardPermissionKey
	 *            the board permission key.
	 * @param agentBody
	 *            the AgentBody of the Agent.
	 * @throws IncorrectKeyException
	 *             if the given permission key is not the good one.
	 */
	public void agentTerminate(long boardPermissionKey, AgentBody agentBody)
			throws IncorrectKeyException;

	/**
	 * An agent wins or loses energy.
	 * 
	 * @param boardPeremissionKey
	 *            the board permission key.
	 * @param agentEnergy
	 *            the AgentEnergy of the Agent.
	 * @param energy
	 *            a positive or negative quantity.
	 * @throws IncorrectKeyException
	 *             if the given permission key is not the good one.
	 */
	public void agentEnergyChange(long boardPermissionKey,
			AgentEnergy agentEnergy, int energy) throws IncorrectKeyException;
	
	/**
	 * An Agent moves to another Station.
	 * 
	 * @param boardPermissionKey
	 *            the board permission key.
	 * @param agentLocation
	 *            the considered AgentLocation.
	 * @param destination
	 *            a destination Station.
	 * @throws IncorrectKeyException
	 *             if the given permission key is not the good one.
	 */
	public void agentMove(long boardPermissionKey, AgentLocation agentLocation,
			Station destination) throws IncorrectKeyException;

	/**
	 * An Agent quits the land.
	 * 
	 * @param boardPermissionKey
	 *            the board permission key.
	 * @param agentLocation
	 *            the considered AgentLocation.
	 * @throws IncorrectKeyException
	 *             if the given permission key is not the good one.
	 */
	public void agentQuit(long boardPermissionKey, AgentLocation agentLocation)
			throws IncorrectKeyException;

	/**
	 * Land end.
	 * 
	 * @param boardPermissionKey
	 *            the board permission key.
	 * @param donePermissionKey
	 *            the done permission key.
	 * @throws IncorrectKeyException
	 *             if a permission key is not the good one.
	 */
	public void done(long boardPermissionKey, long donePermissionKey)
			throws IncorrectKeyException;

	/**
	 * @return true after land end.
	 */
	public boolean isDone();
}
