/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.impl;

import java.util.Arrays;

import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.demo.land.Graph;

/**
 * Graph implementation.
 */
public class GraphImpl implements Graph {
	private static PrintStream trace = null;

	public static void setTrace(PrintStream trace) {
		GraphImpl.trace = trace;
	}

	/**
	 * Creates and returns a toroidal Graph.
	 * 
	 * @param nbRow
	 *            a number of row.
	 * @param nbColumn
	 *            a number of column.
	 * @return the created Graph.
	 */
	public GraphImpl createToroidalGraph(int nbRow, int nbColumn) {
		int stationCount = nbRow * nbColumn;
		int pathCount = 4;
		int[] links = new int[stationCount * pathCount];
		int index = 0;
		for (int r = 0; r < nbRow; r++) {
			int r1 = r - 1;
			int r2 = r + 1;
			if (r1 < 0)
				r1 += nbRow;
			else if (r2 >= nbRow)
				r2 -= nbRow;
			int br = r * nbColumn;
			int br1 = r1 * nbColumn;
			int br2 = r2 * nbColumn;
			for (int c = 0; c < nbColumn; c++) {
				int c1 = c - 1;
				int c2 = c + 1;
				if (c1 < 0)
					c1 += nbColumn;
				else if (c2 >= nbColumn)
					c2 -= nbColumn;
				links[index++] = br + c1;
				links[index++] = br + c2;
				links[index++] = br1 + c;
				links[index++] = br2 + c;
			}
		}
		return new GraphImpl(stationCount, pathCount, links);
	}
	
	final int stationCount;
	final int minPathCount;
	final int maxPathCount;
	private int[] links;
	private int[] pathCounts;
	private int[] pathCountStat;
	
	public GraphImpl(int stationCount, int maxPathCount, int[] links) {
		if (stationCount < 3 || maxPathCount < 3
				|| maxPathCount > stationCount - 1
				|| links.length != stationCount * maxPathCount)
			throw new IllegalArgumentException();
		this.stationCount = stationCount;
		this.maxPathCount = maxPathCount;
		this.links = links.clone();
		this.pathCounts = new int[stationCount];
		this.pathCountStat = new int[maxPathCount];
		Arrays.fill(pathCountStat, 0);
		int minPathCount = maxPathCount;
		int index = 0;
		for (int stationIndex = 0; stationIndex < stationCount; stationIndex++) {
			int pathCount = maxPathCount;
			for (int k = 0; k < maxPathCount; k++)
				if (links[index++] < 0 && pathCount == maxPathCount)
					pathCount = k;
			this.pathCounts[stationIndex] = pathCount;
			this.pathCountStat[pathCount - 1]++;
			if (pathCount < minPathCount)
				minPathCount = pathCount;
		}
		this.minPathCount = minPathCount;
		if (trace != null) {
			StringBuffer sb = new StringBuffer();
			sb.append("PathCounts");
			for (int k = minPathCount - 1; k < maxPathCount; k++) {
				sb.append(" ");
				sb.append(k + 1);
				sb.append(":");
				sb.append(this.pathCountStat[k]);
			}
			trace.println(sb.toString());
		}
	}

	///////////////////////
	// Graph implementation
	///////////////////////
	
	public final int getStationCount() {
		return this.stationCount;
	}

	public final int getMinPathCount() {
		return this.minPathCount;
	}

	public final int getMaxPathCount() {
		return this.maxPathCount;
	}

	public final int getPathCount(int stationIndex) {
		return this.pathCounts[stationIndex];
	}

	public final int getDestinationIndex(int stationIndex, int pathIndex) {
		return this.links[stationIndex * this.maxPathCount + pathIndex];
	}

	public final void getDestinationIndexes(int stationIndex, int[] destinationIndexes) {
		System.arraycopy(this.links, stationIndex * maxPathCount,
				destinationIndexes, 0, maxPathCount);
	}
}
