/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.impl;

import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.demo.land.AgentBody;
import org.xmloperator.reactive.demo.land.AgentEnergy;
import org.xmloperator.reactive.demo.land.AgentLocation;
import org.xmloperator.reactive.demo.land.LandBoard;
import org.xmloperator.reactive.demo.land.Move;
import org.xmloperator.reactive.demo.land.Station;
import org.xmloperator.reactive.exception.IncorrectKeyException;
import org.xmloperator.reactive.impl.BoardImpl;

/**
 * LandBoard implementation.
 */
public class LandBoardImpl extends BoardImpl implements LandBoard {
	private final long donePermissionKey;
	private final StationsImpl stations;
	private boolean isDone = false;
	
	public LandBoardImpl(ReactiveCommunity reactiveCommunity,
			long boardPermissionKey, long donePermissionKey,
			StationsImpl stations) {
		super(reactiveCommunity, boardPermissionKey);
		if (stations == null)
			throw new IllegalArgumentException();
		this.donePermissionKey = donePermissionKey;
		this.stations = stations;
	}

	///////////////////////////
	// LandBoard implementation
	///////////////////////////
	
	public final void agentAppear(long boardPermissionKey,
			AgentBody agentBody) throws IncorrectKeyException {
		if (boardPermissionKey != this.boardPermissionKey)
			throw new IncorrectKeyException();
		((AgentFamilyImpl) agentBody.getAgentFamily()).addMember(agentBody);
	}

	public final void agentTerminate(long boardPermissionKey,
			AgentBody agentBody) throws IncorrectKeyException {
		if (boardPermissionKey != this.boardPermissionKey)
			throw new IncorrectKeyException();
		((AgentFamilyImpl) agentBody.getAgentFamily()).removeMember(agentBody);
	}

	public final void agentEnergyChange(long boardPermissionKey,
			AgentEnergy agentEnergy, int energy) throws IncorrectKeyException {
		if (boardPermissionKey != this.boardPermissionKey)
			throw new IncorrectKeyException();
		agentEnergy.addEnergy(energy);
	}

	public final void agentMove(long boardPermissionKey,
			AgentLocation agentLocation, Station destination)
			throws IncorrectKeyException {
		if (boardPermissionKey != this.boardPermissionKey)
			throw new IncorrectKeyException();
		Move move = agentLocation.move(destination);
		this.stations.move(agentLocation.getAgentBody(), move.fromStationIndex,
				move.toStationIndex);
	}

	public final void agentQuit(long boardPermissionKey,
			AgentLocation agentLocation) throws IncorrectKeyException {
		if (boardPermissionKey != this.boardPermissionKey)
			throw new IncorrectKeyException();
		Move move = agentLocation.quit();
		this.stations.move(agentLocation.getAgentBody(), move.fromStationIndex,
				move.toStationIndex);
	}

	public final void done(long boardPermissionKey, long donePermissionKey)
			throws IncorrectKeyException {
		if (boardPermissionKey != this.boardPermissionKey
				|| donePermissionKey != this.donePermissionKey)
			throw new IncorrectKeyException();
		this.isDone = true;
	}

	public final boolean isDone() {
		return this.isDone;
	}
}
