/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.xmloperator.reactive.demo.land.AgentBody;
import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.NavigationStrategy;
import org.xmloperator.reactive.demo.land.Station;
import org.xmloperator.reactive.demo.land.Stations;
import org.xmloperator.reactive.demo.land.navigation.TargetNavigationStrategies;

/**
 * Station implementation.
 */
final class StationImpl implements Station {
	final StationsImpl stations;
	final int stationIndex;
	private final int[] destinationIndexes;
	private final int pathCount;
	private List<AgentBody> occupantList = new ArrayList<AgentBody>();
	private List<AgentBody> unmodifiableOccupantList = Collections
			.unmodifiableList(this.occupantList);
	private TargetNavigationStrategies targetNavigationStrategies = null;
	final int flowSource;

	StationImpl(StationsImpl stations, int stationIndex, int flowSource) {
		Graph graph = stations.graph;
		if (stationIndex < 0 || stationIndex >= graph.getStationCount()
				|| flowSource < 0)
			throw new IllegalArgumentException();
		this.stations = stations;
		this.stationIndex = stationIndex;
		this.destinationIndexes = new int[graph.getMaxPathCount()];
		graph.getDestinationIndexes(stationIndex, this.destinationIndexes);
		this.pathCount = graph.getPathCount(stationIndex);
		this.flowSource = flowSource;
	}

	/**
	 * Adds an occupant.
	 * @param agentBody a new occupant.
	 */
	final void addOccupant(AgentBody agentBody) {
		if (this.occupantList.contains(agentBody))
			throw new IllegalStateException();
		this.occupantList.add(agentBody);
	}
	
	/**
	 * Removes an occupant.
	 * @param agentBody an occupant that leaves.
	 */
	final void removeOccupant(AgentBody agentBody) {
		if (!this.occupantList.contains(agentBody))
			throw new IllegalStateException();
		this.occupantList.remove(agentBody);
	}
	
	/////////////////////////
	// Station implementation
	/////////////////////////

	public final int getStationIndex() {
		return this.stationIndex;
	}
	
	public final Stations getStations() {
		return this.stations;
	}
	
	public final boolean isConnectedTo(Station station) {
		StationImpl other = (StationImpl) station;
		int destinationIndex = other.stationIndex;
		if (this.stationIndex < 0 || destinationIndex < 0
				|| other.stations != this.stations)
			return false;
		for (int k = 0; k < this.pathCount; k++)
			if (this.destinationIndexes[k] == destinationIndex)
				return true;
		return false;
	}
	
	public final List<AgentBody> getOccupantList() {
		return this.unmodifiableOccupantList;
	}
	
	public final NavigationStrategy getTowardsNavigationStrategy() {
		if (this.targetNavigationStrategies == null)
			this.targetNavigationStrategies = new TargetNavigationStrategies(
					this.stations, this);
		return this.targetNavigationStrategies
				.getTowardsTargetNavigationStrategy();
	}

	public final NavigationStrategy getAwayNavigationStrategy() {
		if (this.targetNavigationStrategies == null)
			this.targetNavigationStrategies = new TargetNavigationStrategies(
					this.stations, this);
		return this.targetNavigationStrategies
				.getAwayTargetNavigationStrategy();
	}

	public int getFlowSource() {
		return this.flowSource;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("S");
		sb.append(this.stationIndex);
		sb.append("[");
		sb.append(this.flowSource);
		sb.append("]");
//		if (!this.occupantList.isEmpty()) {
//			sb.append("{");
//			boolean isStart = true;
//			for (AgentBody agentBody : this.occupantList) {
//				if (isStart)
//					isStart = false;
//				else
//					sb.append(",");
//				sb.append(agentBody);
//			}
//			sb.append("}");
//		}
		return sb.toString();
	}
}
