/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.navigation;

import org.xmloperator.reactive.demo.land.NavigationStrategy;
import org.xmloperator.reactive.demo.land.VisitMemory;

/**
 * Navigation strategy that consists in using a first NavigationStrategy if the
 * move is effective, a second one elsewhere.
 */
public final class BiNavigationStrategy extends
		NavigationStrategyImpl {
	private NavigationStrategy mainNavigationStrategy;
	private NavigationStrategy elseNavigationStategy;
	
	public BiNavigationStrategy(NavigationStrategy mainNavigationStrategy,
			NavigationStrategy elseNavigationStategy) {
		if (mainNavigationStrategy == null || elseNavigationStategy == null)
			throw new IllegalArgumentException();
		this.mainNavigationStrategy = mainNavigationStrategy;
		this.elseNavigationStategy = elseNavigationStategy;
	}

	/////////////////////
	// NavigationStrategy
	/////////////////////
	
	public int nextStationIndex(int stationIndex, VisitMemory visitMemory) {
		int newStationIndex = this.mainNavigationStrategy.nextStationIndex(
				stationIndex, null);
		if (newStationIndex != stationIndex) {
			if (visitMemory != null)
				visitMemory.visit(newStationIndex);
			return newStationIndex;
		}
		return this.elseNavigationStategy.nextStationIndex(stationIndex,
				visitMemory);
	}

	public NavigationStrategy almostClone(long seed) {
		return new BiNavigationStrategy(
				this.mainNavigationStrategy.almostClone(seed),
				this.elseNavigationStategy.almostClone(seed));
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "[" + this.mainNavigationStrategy + "/"
				+ this.elseNavigationStategy + "]";
	}
}
