/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.navigation;

import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.Station;
import org.xmloperator.reactive.demo.land.Stations;
import org.xmloperator.reactive.demo.land.VisitMemory;

/**
 * Navigation strategy that consists in going to the stations with the greater
 * or the lesser number of occupant.
 */
public final class OccupationNavigationStrategy extends NavigationStrategyImpl {
	private final Stations stations;
	private final boolean maximizeOccupation;
	private final boolean isCurrentStationIncluded;
	private final int[] destinationIndexes;

	public OccupationNavigationStrategy(Stations stations,
			boolean maximizeOccupation, boolean isCurrentStationIncluded) {
		if (stations == null)
			throw new IllegalArgumentException();
		this.stations = stations;
		this.maximizeOccupation = maximizeOccupation;
		this.isCurrentStationIncluded = isCurrentStationIncluded;
		this.destinationIndexes = new int[stations.getGraph().getMaxPathCount()];
	}

	public boolean isOccupationMaximized() {
		return this.maximizeOccupation;
	}
	
	////////////////////////////////////
	// NavigationStrategy implementation
	////////////////////////////////////
	
	public int nextStationIndex(int stationIndex, VisitMemory visitMemory) {
		Station station = this.stations.getStation(stationIndex);
		int bestOccupationCount = this.isCurrentStationIncluded
				? station.getOccupantList().size()
				: this.maximizeOccupation ? -1 : Integer.MAX_VALUE;
		int bestOccupationCountIndex = stationIndex;
		Graph graph = this.stations.getGraph();
		int pathCount = graph.getPathCount(stationIndex);
		synchronized (this) {
			graph.getDestinationIndexes(stationIndex, this.destinationIndexes);
			for (int k = 0; k < pathCount; k++) {
				int destinationIndex = this.destinationIndexes[k];
				Station destination = this.stations.getStation(destinationIndex);
				int occupantCount = destination.getOccupantList().size();
				if (this.maximizeOccupation
						? (occupantCount > bestOccupationCount)
						: (occupantCount < bestOccupationCount)) {
					bestOccupationCount = occupantCount;
					bestOccupationCountIndex = destinationIndex;
				}
			}
		}
		if (visitMemory != null)
			visitMemory.visit(bestOccupationCountIndex);
		return bestOccupationCountIndex;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return this.maximizeOccupation ? "crowd" : "alone";
	}
}
