/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.sphere;

import java.util.List;

/**
 * Data about a station that is located on a sphere of radius 1.
 */
public interface Station {

	/**
	 * @return the location of this Station on a sphere of radius 1.
	 */
	public Vector3 getLocation();
	
	/**
	 * @return a first tangent at the station location.
	 */
	public Vector3 getTangentU();

	/**
	 * @return a second tangent, perpendicular to the first one.
	 */
	public Vector3 getTangentV();

	/**
	 * @return the list of path to the other stations, sorted by increasing
	 *         distance.
	 */
	public List<? extends StationPath> getStationPathList();

	/**
	 * @return the resultant vector of the repulsions of the other stations.
	 */
	public Vector3 getRepulsionResultant();
	
	/**
	 * @return the sum of the repulsions of the other stations.
	 */
	public double repulsionSum();
}
