/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.sphere.impl;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Random;

import org.xmloperator.reactive.demo.land.sphere.Vector3;

/**
 * Vector3 Implementation.
 */
public final class Vector3Impl implements Vector3 {
	private static final NumberFormat formatter = new DecimalFormat("0.000");
	
	public static final String toString(double t) {
		String s = formatter.format(t);
		if (t >= 0)
			s = '+' + s;
		return s;
	}
	
	public static final double norm(double x, double y, double z) {
		return Math.sqrt(x * x + y * y + z * z);
	}
	
	final double x;
	final double y;
	final double z;
	
	/**
	 * Basic constructor.
	 * @param x an x coordinate.
	 * @param y an y coordinate.
	 * @param z an z coordinate.
	 */
	public Vector3Impl(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	/**
	 * Constructor of a random Point on a sphere of radius 1.
	 * @param rnd
	 */
	public Vector3Impl(Random rnd) {
		double x, y, z, r;
		do {
			x = rnd.nextDouble() - 0.5;
			y = rnd.nextDouble() - 0.5;
			z = rnd.nextDouble() - 0.5;
			r = norm(x, y, z);
		} while (r < 1e-20);
		this.x = x / r;
		this.y = y / r;
		this.z = z / r;
	}
	
	/////////////////////////
	// Vector3 implementation
	/////////////////////////
	
	public double getX() {
		return this.x;
	}

	public double getY() {
		return this.y;
	}

	public double getZ() {
		return this.z;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "(" + toString(this.x) + "," + toString(this.y) + ","
				+ toString(this.z) + ")";
	}
}
