/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.math.BigInteger;

import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * Account of an agent with an operator.
 */
public interface Account {

	/**
	 * Returns the Resource used by this Account.
	 * @return the Resource used by this Account.
	 */
	public Resource getResource();
	
	/**
	 * Returns the Bank that manages this Account.
	 * @return the Bank that manages this Account.
	 */
	public Bank getBank();

	/**
	 * Returns the maximum debt acceptable by this Account. Negated, it is the
	 * maximum negative sold.
	 * 
	 * @return the maximum debt acceptable by this Account. Negated, it is the
	 *         maximum negative sold.
	 */
	public BigInteger getMaxDebt();
	
	/**
	 * Returns the count of Resource units that represents this Account. A
	 * negative value, if permitted, would represent a debt.
	 * 
	 * @param holderKey
	 *            the access key of the holder of this Account.
	 * @return the count of Resource units that represents this Account.
	 * @throws IncorrectKeyException
	 *                if the holderKey is incorrect.
	 */
	public BigInteger getResourceUnitCount(long holderKey)
			throws IncorrectKeyException;

	/**
	 * Returns the virtual value represented by the content of this Account.
	 * 
	 * @param holderKey
	 *            the access key of the holder of this Account.
	 * @return the virtual value represented by the content of this Account.
	 * @throws IncorrectKeyException
	 *             if the holderKey is incorrect.
	 */
	public double getResourceValue(long holderKey)
			throws IncorrectKeyException;
	
	/**
	 * Transfer a count of Resource units from this Account to a destination
	 * Account.
	 * 
	 * @param holderKey
	 *            the access key of the holder of this Account.
	 * @param destinationAccount
	 *            the destination Account.
	 * @param unitCount
	 *            the count of Resource units.
	 * @return true in case of success, false on failure due to insufficient sold.
	 * @exception IncorrectKeyException
	 *                thrown if the holderKey is incorrect.
	 * @exception DifferentResourcesException
	 *                if the destination Resource is not the source Resource.
	 */
	public boolean transfer(long holderKey,
			Account destinationAccount, BigInteger unitCount)
			throws IncorrectKeyException, DifferentResourcesException;
}
