/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

/**
 * Ingredient within a Transformation.
 */
public interface Ingredient {
	
	/**
	 * Returns the Resource.
	 * @return the Resource.
	 */
	public Resource getResource();

	/**
	 * Returns the Resource unit count. The number is positive for
	 * produced Resource, negative for a consumed Resource.
	 * 
	 * @return the Resource unit count.
	 */
	public int getResourceUnitCount();

	/**
	 * Returns the virtual value of this Ingredient. The number is positive for
	 * produced Resource, negative for a consumed Resource.
	 * 
	 * @return the virtual value of this Ingredient.
	 */
	public double getIngredientValue();

	/**
	 * Returns the ProjectedAccount affected by using this Ingredient.
	 * @return the ProjectedAccount affected by using this Ingredient.
	 */
	public ProjectedAccount getAffectedAccount();
}
