/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.math.BigInteger;

import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * An Account with anticipations.
 */
public interface ProjectedAccount extends Account {

	/**
	 * Returns the Resource count units of this Account minus the sum of
	 * negative anticipations.
	 * 
	 * @param holderKey
	 *            the access key of the holder of this Account.
	 * @return the Resource count units of this Account minus the sum of
	 *         negative Anticipations.
	 * @exception IncorrectKeyException
	 *                thrown if the holderKey is incorrect.
	 */
	public BigInteger getSoldWithNegativeAnticipations(long holderKey)
			throws IncorrectKeyException;
	
	/**
	 * Returns the Resource count units of this Account added to the sum of
	 * positive anticipations.
	 * 
	 * @param holderKey
	 *            the access key of the holder of this Account.
	 * @return the Resource count units of this Account added to the sum of
	 *         positive anticipations.
	 * @exception IncorrectKeyException
	 *                thrown if the holderKey is incorrect.
	 */
	public BigInteger getSoldWithPositiveAnticipations(long holderKey)
			throws IncorrectKeyException;
	/**
	 * Adds a (negative or positive) Anticipation.
	 * @param holderKey the entity of the holder of this Account.
	 * @param anticipation an Anticipation to add.
	 * @exception IncorrectKeyException
	 *                thrown if the holderKey is incorrect.
	 */
	public void addAnticipation(long holderKey,
			Anticipation anticipation) throws IncorrectKeyException;
	
	/**
	 * Removes a (negative or positive) Anticipation.
	 * @param holderKey the entity of the holder of this Account.
	 * @param anticipation an Anticipation to remove.
	 * @exception IncorrectKeyException
	 *                thrown if the holderKey is incorrect.
	 */
	public void removeAnticipation(long holderKey,
			Anticipation anticipation) throws IncorrectKeyException;
	
	/**
	 * Returns true if anticipations are present, false elsewhere.
	 * @return true if anticipations are present, false elsewhere.
	 */
	public boolean hasAnticipations();
}
