/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

/**
 * Something that can be exchanged on a market.
 */
public interface Resource {
	
	/**
	 * Returns the name of this Resource.
	 * @return the name of this Resource.
	 */
	public String getResourceName();
	
	/**
	 * Returns the reference Bank for this Resource.
	 * @return the reference Bank for this Resource.
	 */
	public Bank getReferenceBank();

	/**
	 * Returns the value of a unit of this Resource using a virtual unit. The
	 * virtual unit is only used for evaluation. It is not a reference on the
	 * market.
	 * 
	 * @return the value of a unit of this Resource using a virtual unit.
	 */
	public double getUnitValue();
	
	/**
	 * Returns the weight of this Resource in a basket of resources.
	 * @return the weight of this Resource in a basket of resources.
	 */
	public double getWeight();
}
