/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market;

import java.util.List;

import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * Transformation from a list to resources and another list of resources.
 */
public interface Transformation {

	/**
	 * Returns the name of this Transformation.
	 * @return the name of this Transformation.
	 */
	public String getTransformationName();
	
	/**
	 * Returns the list of (produced or consumed) Ingredient of this
	 * Transformation.
	 * 
	 * @return the list of (produced or consumed) Ingredient of this
	 *         Transformation.
	 */
	public List<? extends Ingredient> getIngredientList();
	
	/**
	 * Returns the Ingredient of the given Resource.
	 * @param resource an Ingredient Resource.
	 * @return the Ingredient of the given Resource.
	 */
	public Ingredient getIngredient(Resource resource);

	/**
	 * Returns the sum of virtual values of the resources contained in the
	 * accounts associated to the ingredients of this Transformation.
	 * 
	 * @param holderKey
	 *            the access key of the holder of the accounts.
	 * @return the sum of virtual values of the resources contained in the
	 *         accounts associated to the ingredients of this Transformation.
	 * @throws IncorrectKeyException
	 *             if the holderKey is incorrect.
	 */
	public double getAccountsValue(long holderKey)
			throws IncorrectKeyException;

	/**
	 * Returns true if some Ingredient of this Transformation has anticipations
	 * on its account.
	 * 
	 * @return true if some Ingredient of this Transformation has anticipations
	 *         on its account.
	 */
	public boolean hasAnticipations();
}
