package org.xmloperator.reactive.demo.market.data;

import java.math.BigInteger;
import java.util.Random;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.ReactiveCommunity;

import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.agent.ReactiveFactoryImpl;

import org.xmloperator.reactive.demo.market.Account;
import org.xmloperator.reactive.demo.market.MarketBoard;
import org.xmloperator.reactive.demo.market.ProjectedAccount;
import org.xmloperator.reactive.demo.market.Resource;
import org.xmloperator.reactive.demo.market.action.OperatorAction;
import org.xmloperator.reactive.demo.market.action.ProposalMessage;
import org.xmloperator.reactive.demo.market.action.TransformAction;
import org.xmloperator.reactive.demo.market.agent.Operator;
import org.xmloperator.reactive.demo.market.agent.Shaker;
import org.xmloperator.reactive.demo.market.agent.Transformer;
import org.xmloperator.reactive.demo.market.impl.IngredientImpl;
import org.xmloperator.reactive.demo.market.impl.MarketBoardImpl;
import org.xmloperator.reactive.demo.market.impl.TransformationImpl;
import org.xmloperator.reactive.demo.market.impl.TransformationsImpl;

import org.xmloperator.reactive.exception.ReactiveException;

/**
 * Market demonstration.
 */
public class MarketDemo extends AgentImpl {

	public MarketDemo() {
		super(false);
	}

	///////////////////////
	// Agent implementation
	///////////////////////
	
	public void participate() {
		ReactiveCommunity community = this.entity.getCommunity();
		Event proposalMessageEvent = community.createMessageEvent();
		Event exitMessageEvent = community.createMessageEvent();
		long initiatorKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
//		ElectricityMarketData marketData = new ElectricityMarketData(initiatorKey);
//		Resource traceableResource = marketData.electricity;
		LemonadeMarketData marketData = new LemonadeMarketData(initiatorKey);
		Resource traceableResource = marketData.lemonade;
		ProposalMessage.setTraceableResource(traceableResource);
		OperatorAction.setTraceableResource(traceableResource);
		TransformAction.setTraceableResource(traceableResource);
		/*
		 * Transformations.
		 */
		Object[][] transformersData = marketData.getTransformers();
		long[] operatorKeys = new long[transformersData.length];
		TransformationImpl[] transformations = new TransformationImpl[transformersData.length];
		for (int transformationIndex = 0; transformationIndex < transformersData.length; transformationIndex++) {
			Object[] transformationData = transformersData[transformationIndex];
			String transformationName = (String) transformationData[0];
			Resource[] resources = (Resource[]) transformationData[1];
			int[] unitCounts = (int[]) transformationData[2];
			if (unitCounts.length != resources.length)
				throw new IllegalStateException();
			long operatorKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
			operatorKeys[transformationIndex] = operatorKey;
			IngredientImpl[] ingredients = new IngredientImpl[resources.length];
			for (int k = 0; k < resources.length; k++) {
				Resource resource = resources[k];
				int unitCount = unitCounts[k];
				ProjectedAccount account = unitCount != 0
						? resource.getReferenceBank().createProjectedAccount(operatorKey)
						: null;
				ingredients[k] = new IngredientImpl(resource,
						unitCount, account);
			}
			TransformationImpl transformation = new TransformationImpl(
					transformationName, ingredients);
			transformations[transformationIndex] = transformation;
		}
		/*
		 * MarketBoard.
		 */
		long permissionKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		MarketBoard marketBoard = new MarketBoardImpl(community,
				permissionKey, new TransformationsImpl(transformations));
		community.setBoard(marketBoard, permissionKey);
		/*
		 * Transformers.
		 */
		for (int transformationIndex = 0; transformationIndex < transformersData.length; transformationIndex++) {
			Object[] transformationData = transformersData[transformationIndex];
			Resource[][] missionsData = (Resource[][]) transformationData[3];
			BigInteger maxTransformationFactor = (BigInteger) transformationData[4];
			Resource capitalResource = (Resource) transformationData[5];
			BigInteger capitalUnitCount = (BigInteger) transformationData[6];
			long operatorKey = operatorKeys[transformationIndex];
			TransformationImpl transformation = transformations[transformationIndex];
			Transformer transformer = new Transformer(operatorKey,
					proposalMessageEvent, exitMessageEvent,
					transformation, maxTransformationFactor);
			for (Resource[] missionData : missionsData) {
				Resource proposedResource = missionData[0];
				Resource purchasedResource = missionData[1];
				ProjectedAccount proposedResourceAccount = transformation
						.getIngredient(proposedResource)
						.getAffectedAccount();
				if (proposedResourceAccount == null)
					throw new ReactiveException(proposedResource + " is not a transformed Resource");
				ProjectedAccount purchasedResourceAccount = transformation
						.getIngredient(purchasedResource)
						.getAffectedAccount();
				if (purchasedResourceAccount == null)
					throw new ReactiveException(purchasedResource + " is not a transformed Resource");
				transformer.addMission(purchasedResourceAccount, proposedResourceAccount);
			}
			{
				/*
				 * Transformer capital.
				 */
				Account capitalAccount = capitalResource
						.getReferenceBank().createReceivableAccount(
								initiatorKey, operatorKey,
								capitalUnitCount);
				Account creditAccount = transformation.getIngredient(
						capitalResource).getAffectedAccount();
				marketBoard
						.newTransfer(permissionKey, operatorKey,
								capitalAccount, creditAccount,
								capitalUnitCount).execute();
			}
			community.launchAgent(transformer);
		}
		/*
		 * Operators.
		 */
		Object[][] operatorsData = marketData.getOperators();
		for (Object[] operatorData : operatorsData) {
			Resource againstResource = (Resource) operatorData[0];
			Resource proposedResource = (Resource) operatorData[1];
			BigInteger capitalUnitCount = (BigInteger) operatorData[2];
			long operatorKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
			ProjectedAccount proposedResourceAccount = proposedResource
					.getReferenceBank().createProjectedAccount(
							operatorKey);
			ProjectedAccount againstResourceAccount = againstResource
					.getReferenceBank().createProjectedAccount(
							operatorKey);
			Operator operator = new Operator(marketBoard,
					operatorKey, proposedResourceAccount, againstResourceAccount,
					proposalMessageEvent, exitMessageEvent);
			{
				/*
				 * Operator capital.
				 */
				Account capitalAccount = againstResource
						.getReferenceBank().createReceivableAccount(
								initiatorKey, operatorKey,
								capitalUnitCount);
				marketBoard.newTransfer(permissionKey, operatorKey,
						capitalAccount, againstResourceAccount,
						capitalUnitCount).execute();
			}
			community.launchAgent(operator);
		}
		/*
		 * Shaker.
		 */
		Shaker shaker = new Shaker(exitMessageEvent, new Random());
		community.launchAgent(shaker);
		/*
		 * Run.
		 */
		for (int k = 0; k < 10; k++)
			this.entity.stop();
		this.entity.generateMessage(exitMessageEvent, null);
	}
}
