/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.agent;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.agent.ReactiveFactoryImpl;

import org.xmloperator.reactive.element.ElementBoard;
import org.xmloperator.reactive.element.RootElement;
import org.xmloperator.reactive.element.impl.ElementBoardImpl;
import org.xmloperator.reactive.element.util.DocumentReader;
import org.xmloperator.reactive.element.util.DocumentWriter;

import org.xmloperator.reactive.exception.ReactiveException;

/**
 * Implementation of Agent that manages an ElementBoard.
 */
public abstract class ElementBoardAgent extends AgentImpl {

	protected final String boardFileName;
	protected RootElement rootElement = null;
	
	public ElementBoardAgent(String boardFileName) {
		super(false);
		if (boardFileName == null)
			throw new IllegalArgumentException();
		this.boardFileName = boardFileName;
	}

	///////////////////////
	// Agent implementation
	///////////////////////

	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		ReactiveCommunity community = launchingContext.getReactiveCommunity();
		PrintStream trace = launchingContext.getTrace();
		long writingKey = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		ElementBoard elementBoard = new ElementBoardImpl(community, writingKey);
		try {
			DocumentReader.readDocument(new FileInputStream(boardFileName),
					elementBoard.getRootElement(), elementBoard, writingKey);
			if (trace != null)
				trace.println(community + " board read on \"" + boardFileName
						+ "\"");
		} catch (Exception exception) {
			exception.printStackTrace();
			elementBoard = new ElementBoardImpl(community, writingKey);
			if (trace != null)
				trace.println(community + " board initialized");
		}
		community.setBoard(elementBoard, writingKey);
		this.rootElement = elementBoard.getRootElement();

		// the board is available at the next instant.
		this.entity.stop();
	}

	public void terminate() {
		super.terminate();
		PrintStream trace = launchingContext.getTrace();
		try {
			DocumentWriter.printDocument(new FileOutputStream(this.boardFileName),
					this.rootElement, null);
			if (trace != null) {
				ReactiveCommunity community = launchingContext.getReactiveCommunity();
				trace.println(community + " board saved on \"" + boardFileName
						+ "\"");
			}
		} catch (FileNotFoundException exception) {
			throw new ReactiveException("Unable to save the board on "
					+ boardFileName);
		}
	}
}
