/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.exception;

import org.xmloperator.reactive.exception.ReactiveException;

/**
 * This ReactiveException is thrown when a name is proposed with bad chars.
 * Name ::= ([a-zA-Z] | '_') ([a-zA-Z0-9] | '_' | '-' | '.')*
 */
public final class BadCharInNameException extends ReactiveException {

	private static final long serialVersionUID = 1479959449167255398L;

	public static void verifyName(String name) {
		int length = name.length();
		boolean ok = (length != 0);
		for (int k = 0; k < length; k++)
			if (!isNameChar(name.charAt(k), k == 0))
				ok = false;
		if (!ok)
			throw new BadCharInNameException();
	}
	
	private static boolean isNameChar(int ch, boolean start) {
		// Name ::= ([a-zA-Z] | '_') ([a-zA-Z0-9] | '_' | '-' | '.')*
		return (ch >= 'a' && ch <= 'z')
			|| (ch >= 'A' && ch <= 'Z')
			|| (ch == '_')
			|| (!start
				&& (
					(ch >= '0' && ch <= '9')
					|| (ch == '-')
					|| (ch == '.')));
	}

	public BadCharInNameException() {
	}
}
