/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.impl;


import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.DataElement;
import org.xmloperator.reactive.element.DoubleDataElement;
import org.xmloperator.reactive.element.exception.BadCharInNameException;

/**
 * DoubleDataElement implementation.
 */
public final class DoubleDataElementImpl extends ElementImpl implements DoubleDataElement {

	final double data;

	public DoubleDataElementImpl(String namespaceURI, String tagName,
			double data) throws BadCharInNameException {
		super(namespaceURI, tagName);
		this.data = data;
	}

	///////////////////////////////////
	// DoubleDataElement implementation
	///////////////////////////////////
	
	public double getData() {
		return this.data;
	}

	/////////////////////////////
	// DataElement implementation
	/////////////////////////////
	
	public String getDataType() {
		return DataElement.TYPE_DOUBLE;
	}
	
	public String stringDataValue() {
		return Double.toString(this.data);
	}
	
	/////////////////////////
	// Element implementation
	/////////////////////////
	
	public boolean isDataElement() {
		return true;
	}

	public boolean equals(Element element) {
		if (element == this)
			return true;
		if (!(element instanceof DoubleDataElementImpl)
				|| !super.equals(element))
			return false;
		DoubleDataElementImpl otherElement = (DoubleDataElementImpl)element;
		return this.data == otherElement.data;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append('(');
		sb.append(this.data);
		sb.append(')');
		return sb.toString();
	}
}
