/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import org.xmloperator.reactive.Action;
import org.xmloperator.reactive.ActionMessage;

/**
 * ActionMessage implementation.
 */
final class ActionMessageImpl extends MessageImpl implements ActionMessage {
	private final Action action;
	private Exception exception = null;
	
	ActionMessageImpl(Action action) {
		if (action == null)
			throw new IllegalArgumentException();
		this.action = action;
	}
	
	void setException(Exception exception) {
		if (exception == null)
			throw new IllegalArgumentException();
		if (this.exception != null)
			throw new IllegalStateException();
		this.exception = exception;
	}
	
	///////////////////////////////
	// ActionMessage implementation
	///////////////////////////////
	
	public final Action getAction() {
		return this.action;
	}

	public final Exception getException() {
		return this.exception;
	}

	/////////////////////////
	// Message implementation
	/////////////////////////
	
	public final boolean isActionMessage() {
		return true;
	}

	public final boolean isTraceable() {
		return this.action.isTraceable();
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append('(');
		sb.append(this.action);
		if (this.exception != null) {
			sb.append(',');
			sb.append(exception);
		}
		sb.append(')');
		return sb.toString();
	}
}
